@@ -17,10 +17,10 @@ import abc
 import mock
 from oslotest import base
 import six
-from six import moves
 from six.moves.urllib import parse as url_parse
 
 from ceilometer.network.statistics.opendaylight import driver
+from ceilometer import service
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -76,8 +76,8 @@ class _Base(base.BaseTestCase):
     def setUp(self):
         super(_Base, self).setUp()
         self.addCleanup(mock.patch.stopall)
-
-        self.driver = driver.OpenDayLightDriver()
+        conf = service.prepare_service([], [])
+        self.driver = driver.OpenDayLightDriver(conf)
 
         self.get_flow_statistics = mock.patch(
             'ceilometer.network.statistics.opendaylight.client.'
@@ -118,10 +118,7 @@ class _Base(base.BaseTestCase):
                                                   self.fake_params,
                                                   {})
 
-        for sample, expected in moves.zip(sample_data, expected_data):
-            self.assertEqual(expected[0], sample[0])  # check volume
-            self.assertEqual(expected[1], sample[1])  # check resource id
-            self.assertEqual(expected[2], sample[2])  # check resource metadata
+        self.assertEqual(expected_data, list(sample_data))
 
 
 class TestOpenDayLightDriverSpecial(_Base):
@@ -135,6 +132,23 @@ class TestOpenDayLightDriverSpecial(_Base):
     active_hosts_data = {"hostConfig": []}
     inactive_hosts_data = {"hostConfig": []}
 
+    def test_dict_to_kv(self):
+        data = {'a': 'A',
+                'b': 'B',
+                'nested': {'a': 'A',
+                           'b': 'B',
+                           },
+                'nested2': [{'c': 'A'}, {'c': 'B'}]
+                }
+        pairs = list(self.driver.dict_to_keyval(data))
+        self.assertEqual([('a', 'A'),
+                          ('b', 'B'),
+                         ('nested.a', 'A'),
+                         ('nested.b', 'B'),
+                         ('nested2[0].c', 'A'),
+                         ('nested2[1].c', 'B')],
+                         sorted(pairs, key=lambda x: x[0]))
+
     def test_not_implemented_meter(self):
         sample_data = self.driver.get_sample_data('egg',
                                                   self.fake_odl_url,
@@ -351,7 +365,7 @@ class TestOpenDayLightDriverSimple(_Base):
                 'container': 'default',
                 "properties_actions": "4095",
                 "properties_timeStamp_connectedSince": "1377291227877"
-            }),
+            }, None),
         ]
 
         self._test_for_meter('switch', expected_data)
@@ -362,7 +376,7 @@ class TestOpenDayLightDriverSimple(_Base):
                 'controller': 'OpenDaylight',
                 'container': 'default',
                 'port': '4',
-            }),
+            }, None),
         ]
         self._test_for_meter('switch.port', expected_data)
 
@@ -371,7 +385,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.receive.packets', expected_data)
 
@@ -380,7 +394,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.transmit.packets', expected_data)
 
@@ -389,7 +403,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.receive.bytes', expected_data)
 
@@ -398,7 +412,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.transmit.bytes', expected_data)
 
@@ -407,7 +421,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.receive.drops', expected_data)
 
@@ -416,7 +430,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.transmit.drops', expected_data)
 
@@ -425,7 +439,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.receive.errors', expected_data)
 
@@ -434,7 +448,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.transmit.errors', expected_data)
 
@@ -443,7 +457,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.receive.frame_error', expected_data)
 
@@ -452,7 +466,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.receive.overrun_error',
                              expected_data)
@@ -462,7 +476,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.receive.crc_error', expected_data)
 
@@ -471,7 +485,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
         ]
         self._test_for_meter('switch.port.collision.count', expected_data)
 
@@ -480,7 +494,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
         ]
         self._test_for_meter('switch.table', expected_data)
 
@@ -489,7 +503,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (11, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
         ]
         self._test_for_meter('switch.table.active.entries', expected_data)
 
@@ -498,7 +512,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (816, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
         ]
         self._test_for_meter('switch.table.lookup.packets', expected_data)
 
@@ -507,7 +521,7 @@ class TestOpenDayLightDriverSimple(_Base):
             (220, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
         ]
         self._test_for_meter('switch.table.matched.packets', expected_data)
 
@@ -531,7 +545,7 @@ class TestOpenDayLightDriverSimple(_Base):
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
                 "flow_priority": "1"
-            }),
+            }, None),
         ]
         self._test_for_meter('switch.flow', expected_data)
 
@@ -554,7 +568,7 @@ class TestOpenDayLightDriverSimple(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.duration_seconds', expected_data)
 
@@ -577,7 +591,7 @@ class TestOpenDayLightDriverSimple(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.duration_nanoseconds', expected_data)
 
@@ -600,7 +614,7 @@ class TestOpenDayLightDriverSimple(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.packets', expected_data)
 
@@ -623,7 +637,7 @@ class TestOpenDayLightDriverSimple(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.bytes', expected_data)
 
@@ -1077,7 +1091,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "properties_macAddress": "00:00:00:00:00:02",
                 "properties_tables": "-1",
                 "properties_timeStamp_connectedSince": "1377291227877"
-            }),
+            }, None),
             (1, "00:00:00:00:00:00:00:03", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1088,7 +1102,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "properties_macAddress": "00:00:00:00:00:03",
                 "properties_tables": "10",
                 "properties_timeStamp_connectedSince": "1377291228000"
-            }),
+            }, None),
         ]
 
         self._test_for_meter('switch', expected_data)
@@ -1099,7 +1113,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 'controller': 'OpenDaylight',
                 'container': 'default',
                 'port': '4',
-            }),
+            }, None),
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1108,7 +1122,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 'user_link_node_port': '5',
                 'user_link_status': 'Success',
                 'user_link_name': 'link1',
-            }),
+            }, None),
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1120,7 +1134,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "topology_name": "s2-eth3",
                 "topology_state": 1,
                 "topology_timeStamp_creation": 1379527162648
-            }),
+            }, None),
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1130,7 +1144,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 'host_networkAddress': '2.2.2.2',
                 'host_staticHost': 'true',
                 'host_vlan': '0',
-            }),
+            }, None),
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1140,7 +1154,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 'host_networkAddress': '2.2.2.4',
                 'host_staticHost': 'false',
                 'host_vlan': '1',
-            }),
+            }, None),
         ]
         self._test_for_meter('switch.port', expected_data)
 
@@ -1149,23 +1163,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (182, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (174, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.receive.packets', expected_data)
 
@@ -1174,23 +1188,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (173, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (181, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.transmit.packets', expected_data)
 
@@ -1199,23 +1213,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (12740, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (12180, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.receive.bytes', expected_data)
 
@@ -1224,23 +1238,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (12110, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (12670, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.transmit.bytes', expected_data)
 
@@ -1249,23 +1263,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.receive.drops', expected_data)
 
@@ -1274,23 +1288,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.transmit.drops', expected_data)
 
@@ -1299,23 +1313,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.receive.errors', expected_data)
 
@@ -1324,23 +1338,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.transmit.errors', expected_data)
 
@@ -1349,23 +1363,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.receive.frame_error', expected_data)
 
@@ -1374,23 +1388,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.receive.overrun_error',
                              expected_data)
@@ -1400,23 +1414,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.receive.crc_error', expected_data)
 
@@ -1425,23 +1439,23 @@ class TestOpenDayLightDriverComplex(_Base):
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '4'}),
+                'port': '4'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '3'}),
+                'port': '3'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '2'}),
+                'port': '2'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '1'}),
+                'port': '1'}, None),
             (0, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'port': '0'}),
+                'port': '0'}, None),
         ]
         self._test_for_meter('switch.port.collision.count', expected_data)
 
@@ -1450,11 +1464,11 @@ class TestOpenDayLightDriverComplex(_Base):
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '1'}),
+                'table_id': '1'}, None),
         ]
         self._test_for_meter('switch.table', expected_data)
 
@@ -1463,11 +1477,11 @@ class TestOpenDayLightDriverComplex(_Base):
             (11, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
             (20, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '1'}),
+                'table_id': '1'}, None),
         ]
         self._test_for_meter('switch.table.active.entries', expected_data)
 
@@ -1476,11 +1490,11 @@ class TestOpenDayLightDriverComplex(_Base):
             (816, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
             (10, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '1'}),
+                'table_id': '1'}, None),
         ]
         self._test_for_meter('switch.table.lookup.packets', expected_data)
 
@@ -1489,11 +1503,11 @@ class TestOpenDayLightDriverComplex(_Base):
             (220, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '0'}),
+                'table_id': '0'}, None),
             (5, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
-                'table_id': '1'}),
+                'table_id': '1'}, None),
         ]
         self._test_for_meter('switch.table.matched.packets', expected_data)
 
@@ -1517,7 +1531,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
                 "flow_priority": "1"
-            }),
+            }, None),
             (1, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1536,7 +1550,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
                 "flow_priority": "1"
-            }),
+            }, None),
         ]
         self._test_for_meter('switch.flow', expected_data)
 
@@ -1559,7 +1573,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
             (5648, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1577,7 +1591,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.duration_seconds', expected_data)
 
@@ -1600,7 +1614,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
             (200000, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1618,7 +1632,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.duration_nanoseconds', expected_data)
 
@@ -1641,7 +1655,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
             (30, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1659,7 +1673,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.packets', expected_data)
 
@@ -1682,7 +1696,7 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
             (89, "00:00:00:00:00:00:00:02", {
                 'controller': 'OpenDaylight',
                 'container': 'default',
@@ -1700,6 +1714,6 @@ class TestOpenDayLightDriverComplex(_Base):
                 "flow_actions_port_type": "OF",
                 "flow_hardTimeout": "0",
                 "flow_idleTimeout": "0",
-                "flow_priority": "1"}),
+                "flow_priority": "1"}, None),
         ]
         self._test_for_meter('switch.flow.bytes', expected_data)
