@@ -16,17 +16,16 @@
 """
 
 import datetime
-import socket
 
 import mock
 import msgpack
-from oslo_config import fixture as fixture_config
 from oslo_utils import netutils
 from oslotest import base
 
 from ceilometer.publisher import udp
 from ceilometer.publisher import utils
 from ceilometer import sample
+from ceilometer import service
 
 
 COUNTER_SOURCE = 'testsource'
@@ -110,31 +109,17 @@ class TestUDPPublisher(base.BaseTestCase):
 
     def setUp(self):
         super(TestUDPPublisher, self).setUp()
-        self.CONF = self.useFixture(fixture_config.Config()).conf
+        self.CONF = service.prepare_service([], [])
         self.CONF.publisher.telemetry_secret = 'not-so-secret'
 
-    def _check_udp_socket(self, url, expected_addr_family):
-        with mock.patch.object(socket, 'socket') as mock_socket:
-            udp.UDPPublisher(netutils.urlsplit(url))
-            mock_socket.assert_called_with(expected_addr_family,
-                                           socket.SOCK_DGRAM)
-
-    def test_publisher_udp_socket_ipv4(self):
-        self._check_udp_socket('udp://127.0.0.1:4952',
-                               socket.AF_INET)
-
-    def test_publisher_udp_socket_ipv6(self):
-        self._check_udp_socket('udp://[::1]:4952',
-                               socket.AF_INET6)
-
     def test_published(self):
         self.data_sent = []
         with mock.patch('socket.socket',
                         self._make_fake_socket(self.data_sent)):
             publisher = udp.UDPPublisher(
+                self.CONF,
                 netutils.urlsplit('udp://somehost'))
-        publisher.publish_samples(None,
-                                  self.test_data)
+        publisher.publish_samples(self.test_data)
 
         self.assertEqual(5, len(self.data_sent))
 
@@ -145,8 +130,7 @@ class TestUDPPublisher(base.BaseTestCase):
             sent_counters.append(counter)
 
             # Check destination
-            self.assertEqual(('somehost',
-                              self.CONF.collector.udp_port), dest)
+            self.assertEqual(('somehost', 4952), dest)
 
         # Check that counters are equal
         def sort_func(counter):
@@ -171,6 +155,6 @@ class TestUDPPublisher(base.BaseTestCase):
         with mock.patch('socket.socket',
                         self._make_broken_socket):
             publisher = udp.UDPPublisher(
+                self.CONF,
                 netutils.urlsplit('udp://localhost'))
-        publisher.publish_samples(None,
-                                  self.test_data)
+        publisher.publish_samples(self.test_data)
