@@ -12,30 +12,30 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import fixtures
 import glanceclient
 import mock
 import novaclient
-from oslo_config import fixture as fixture_config
 from oslotest import base
-from oslotest import mockpatch
 
 from ceilometer import nova_client
+from ceilometer import service
 
 
 class TestNovaClient(base.BaseTestCase):
 
     def setUp(self):
         super(TestNovaClient, self).setUp()
+        self.CONF = service.prepare_service([], [])
         self._flavors_count = 0
         self._images_count = 0
-        self.nv = nova_client.Client()
-        self.useFixture(mockpatch.PatchObject(
+        self.nv = nova_client.Client(self.CONF)
+        self.useFixture(fixtures.MockPatchObject(
             self.nv.nova_client.flavors, 'get',
             side_effect=self.fake_flavors_get))
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             self.nv.glance_client.images, 'get',
             side_effect=self.fake_images_get))
-        self.CONF = self.useFixture(fixture_config.Config()).conf
 
     def fake_flavors_get(self, *args, **kwargs):
         self._flavors_count += 1
@@ -242,8 +242,3 @@ class TestNovaClient(base.BaseTestCase):
         self.assertIsNone(instance.kernel_id)
         self.assertIsNone(instance.image)
         self.assertIsNone(instance.ramdisk_id)
-
-    def test_with_nova_http_log_debug(self):
-        self.CONF.set_override("nova_http_log_debug", True)
-        self.nv = nova_client.Client()
-        self.assertIsNotNone(self.nv.nova_client.client.logger)
